from time import sleep

import boto3


def terminate_batch_job(job_arn: str) -> None:
    client = boto3.client("batch")
    client.terminate_job(jobId=job_arn, reason="Prefect workflow termination")
    sleep(60)



import docker


def submit_docker_job(job_definition: dict, volume: docker.models.volumes.Volume) -> str:
    client = docker.APIClient(base_url="unix://var/run/docker.sock")
    host_config = client.create_host_config(binds={volume.name: {"bind": "/mnt", "mode": "rw"}})

    container = client.create_container(**job_definition, host_config=host_config)
    container_id = container.get("Id")

    client.start(container=container_id)

    return container_id



import boto3
from deepdiff import DeepDiff


def register_batch_job(job_definition: dict) -> str:
    client = boto3.client("batch")

    response = client.describe_job_definitions(
        jobDefinitionName=job_definition["jobDefinitionName"],
    )

    if len(response["jobDefinitions"]) > 0:
        existing_definition = sorted(response["jobDefinitions"], key=lambda d: d["revision"]).pop()
        diff = DeepDiff(job_definition, existing_definition, ignore_order=True)

        if "values_changed" not in diff:
            return existing_definition["jobDefinitionArn"]

    response = client.register_job_definition(**job_definition)
    return response["jobDefinitionArn"]



import boto3
from deepdiff import DeepDiff


def register_fargate_task(task_definition: dict) -> str:
    client = boto3.client("ecs")
    response = client.list_task_definitions(familyPrefix=task_definition["family"])

    if len(response["taskDefinitionArns"]) > 0:
        response = client.describe_task_definition(taskDefinition=task_definition["family"])
        existing_definition = response["taskDefinition"]
        diff = DeepDiff(task_definition, existing_definition, ignore_order=True)

        if "values_changed" not in diff:
            return existing_definition["taskDefinitionArn"]

    response = client.register_task_definition(**task_definition)
    return response["taskDefinition"]["taskDefinitionArn"]



import boto3


def submit_batch_job(
    name: str, job_definition_arn: str, user: str, queue: str, size: int
) -> list[str]:
    job_submission = {
        "jobName": f"{user}_{name}",
        "jobQueue": queue,
        "jobDefinition": job_definition_arn,
    }

    if size > 1:
        job_submission["arrayProperties"] = {"size": size}  # type: ignore

    client = boto3.client("batch")
    response = client.submit_job(**job_submission)

    if size > 1:
        return [f"{response['jobArn']}:{i}" for i in range(size)]

    return [response["jobArn"]]



import boto3


def terminate_fargate_task(cluster: str, task_arn: str) -> None:
    client = boto3.client("ecs")
    client.stop_task(cluster=cluster, task=task_arn, reason="Prefect workflow termination")



import re

import pandas as pd
from tabulate import tabulate


def summarize_manifest_files(
    manifest: pd.DataFrame, name: str, conditions: list[dict], seeds: list[int]
) -> str:
    condition_keys = [f"{name}_{condition['key']}" for condition in conditions]
    manifest_keys = manifest.set_index("KEY").filter(regex=f"^{name}", axis="index").reset_index()
    extensions = manifest_keys["EXTENSION"].unique()

    counts = pd.DataFrame(index=condition_keys, columns=extensions).fillna(0)
    summary = pd.DataFrame(index=condition_keys, columns=extensions)

    for entry in manifest_keys.to_dict("records"):
        key = entry["KEY"]
        extension = entry["EXTENSION"]

        match = re.search("[0-9]{4}", key)
        if match:
            key = entry["KEY"].replace(f"_{match.group(0)}", "")
            if key in condition_keys and int(match.group(0)) in seeds:
                counts.loc[key][extension] += 1
                count = counts.loc[key][extension]
                percent = count / len(seeds) * 100
                summary.loc[key, extension] = f"{count}/{len(seeds)} ({percent:.2f} %)"
        elif key in condition_keys:
            summary.loc[key, extension] = "✓"

    summary.dropna(axis=1, how="all", inplace=True)
    summary.fillna("", inplace=True)

    return tabulate(summary, headers="keys", tablefmt="mixed_outline")



import docker


def create_docker_volume(path: str) -> docker.models.volumes.Volume:
    client = docker.DockerClient(base_url="unix://var/run/docker.sock")
    volume = client.volumes.create(
        driver="local", driver_opts={"type": "none", "device": path, "o": "bind"}
    )
    return volume



from typing import Optional


def make_batch_job(
    name: str,
    image: str,
    vcpus: int,
    memory: int,
    environment: Optional[list[dict[str, str]]] = None,
    job_role_arn: Optional[str] = None,
) -> dict:
    container_properties = {
        "image": image,
        "vcpus": vcpus,
        "memory": memory,
    }

    if environment is not None:
        container_properties["environment"] = environment

    if job_role_arn is not None:
        container_properties["jobRoleArn"] = job_role_arn

    return {
        "jobDefinitionName": name,
        "type": "container",
        "containerProperties": container_properties,
    }



def make_fargate_task(
    name: str,
    image: str,
    account: str,
    region: str,
    user: str,
    vcpus: int,
    memory: int,
) -> dict:
    return {
        "containerDefinitions": [
            {
                "name": f"{user}_{name}",
                "image": f"{account}.dkr.ecr.{region}.amazonaws.com/{user}/{image}",
                "essential": True,
                "portMappings": [],
                "environment": [],
                "mountPoints": [],
                "volumesFrom": [],
                "logConfiguration": {
                    "logDriver": "awslogs",
                    "options": {
                        "awslogs-group": f"/ecs/{user}_{name}",
                        "awslogs-region": region,
                        "awslogs-stream-prefix": "ecs",
                        "awslogs-create-group": "true",
                    },
                },
            }
        ],
        "family": f"{user}_{name}",
        "networkMode": "awsvpc",
        "requiresCompatibilities": ["FARGATE"],
        "taskRoleArn": f"arn:aws:iam::{account}:role/BatchJobRole",
        "executionRoleArn": f"arn:aws:iam::{account}:role/ecsTaskExecutionRole",
        "cpu": str(vcpus),
        "memory": str(memory),
    }



from typing import Optional

import docker


def run_docker_command(
    image: str,
    command: list[str],
    volume: Optional[docker.models.volumes.Volume] = None,
    environment: Optional[list] = None,
    detach: bool = False,
) -> None:
    environment = [] if environment is None else environment
    volumes = {} if volume is None else {volume.name: {"bind": "/mnt", "mode": "rw"}}

    client = docker.DockerClient(base_url="unix://var/run/docker.sock")
    client.containers.run(
        image,
        command,
        environment=environment,
        volumes=volumes,
        auto_remove=True,
        detach=detach,
    )



import boto3


def submit_fargate_task(
    name: str,
    task_definition_arn: str,
    user: str,
    cluster: str,
    security_groups: list[str],
    subnets: list[str],
    command: list[str],
) -> list[str]:
    task_submission = {
        "taskDefinition": task_definition_arn,
        "capacityProviderStrategy": [
            {"capacityProvider": "FARGATE", "weight": 1},
            {"capacityProvider": "FARGATE_SPOT", "weight": 1},
        ],
        "cluster": cluster,
        "platformVersion": "LATEST",
        "count": 1,
        "networkConfiguration": {
            "awsvpcConfiguration": {
                "subnets": subnets,
                "assignPublicIp": "ENABLED",
                "securityGroups": security_groups,
            }
        },
        "overrides": {"containerOverrides": [{"name": f"{user}_{name}", "command": command}]},
    }

    client = boto3.client("ecs")
    response = client.run_task(**task_submission)

    return response["tasks"][0]["taskArn"]



from time import sleep
from typing import Union

import boto3
import prefect
from prefect.server.schemas.states import Failed, State

RETRIES_EXCEEDED_EXIT_CODE = 80


def check_fargate_task(cluster: str, task_arn: str, max_retries: int) -> Union[int, State]:
    task_run = prefect.context.get_run_context().task_run  # type: ignore

    if task_run.run_count > max_retries:
        return RETRIES_EXCEEDED_EXIT_CODE

    client = boto3.client("ecs")
    response = client.describe_tasks(cluster=cluster, tasks=[task_arn])["tasks"]

    # Task responses are not immediately available. Wait until available.
    while len(response) != 1:
        sleep(10)
        response = client.describe_tasks(cluster=cluster, tasks=[task_arn])["tasks"]

    status = response[0]["lastStatus"]

    # Wait until task is running or stopped.
    while status not in ("RUNNING", "STOPPED"):
        sleep(10)
        response = client.describe_tasks(cluster=cluster, tasks=[task_arn])["tasks"]
        status = response[0]["lastStatus"]

    if status == "RUNNING":
        return Failed()

    exitcode = response[0]["containers"][0]["exitCode"]
    return exitcode
